@@ -2,8 +2,7 @@
  *
  * SPDX-License-Identifier: Apache-2.0 */
 
-#ifndef __UTIL_ALGORITHM_H__
-#define __UTIL_ALGORITHM_H__
+#pragma once
 
 #include <algorithm>
 
@@ -13,7 +12,6 @@ using std::remove;
 using std::sort;
 using std::stable_sort;
 using std::swap;
+using std::upper_bound;
 
 CCL_NAMESPACE_END
-
-#endif /* __UTIL_ALGORITHM_H__ */
