@@ -75,7 +75,7 @@ void BKE_tracking_settings_init(struct MovieTracking *tracking);
  * Get transformation matrix for a given object which is used
  * for parenting motion tracker reconstruction to 3D world.
  */
-void BKE_tracking_get_camera_object_matrix(struct Object *camera_object, float mat[4][4]);
+void BKE_tracking_get_camera_object_matrix(const struct Object *camera_object, float mat[4][4]);
 /**
  * Get projection matrix for camera specified by given tracking object
  * and frame number.
@@ -525,12 +525,17 @@ struct ImBuf *BKE_tracking_distort_frame(struct MovieTracking *tracking,
                                          int calibration_height,
                                          float overscan);
 
-void BKE_tracking_max_distortion_delta_across_bound(struct MovieTracking *tracking,
-                                                    int image_width,
-                                                    int image_height,
-                                                    struct rcti *rect,
-                                                    bool undistort,
-                                                    float delta[2]);
+/* Given the size of an image that will be distorted/undistorted by the given tracking, compute the
+ * number of pixels that the image will grow/shrink by in each of the four bounds of the image as a
+ * result of the distortion/undistortion. The deltas for the bounds are positive for expansion and
+ * negative for shrinking. */
+void BKE_tracking_distortion_bounds_deltas(MovieTracking *tracking,
+                                           const int size[2],
+                                           const bool undistort,
+                                           int *r_right,
+                                           int *r_left,
+                                           int *r_bottom,
+                                           int *r_top);
 
 /* --------------------------------------------------------------------
  * Image sampling.
