@@ -15,24 +15,25 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-#include "BLI_blenlib.h"
 #include "BLI_ghash.h"
 #include "BLI_map.hh"
 #include "BLI_math_matrix.h"
 #include "BLI_math_vector.h"
+#include "BLI_string.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
-#include "BKE_action.h"
-#include "BKE_anim_data.h"
+#include "BKE_action.hh"
+#include "BKE_anim_data.hh"
 #include "BKE_animsys.h"
-#include "BKE_armature.h"
+#include "BKE_armature.hh"
 #include "BKE_constraint.h"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_fcurve_driver.h"
-#include "BKE_layer.h"
-#include "BKE_main.h"
-#include "BKE_report.h"
+#include "BKE_idprop.hh"
+#include "BKE_layer.hh"
+#include "BKE_main.hh"
+#include "BKE_report.hh"
 
 #include "DEG_depsgraph.hh"
 #include "DEG_depsgraph_build.hh"
@@ -51,9 +52,11 @@
 #include "UI_interface.hh"
 #include "UI_resources.hh"
 
-#include "ANIM_bone_collections.h"
+#include "ANIM_bone_collections.hh"
 
-#include "armature_intern.h"
+#include "armature_intern.hh"
+
+using blender::Vector;
 
 /* -------------------------------------------------------------------- */
 /** \name Edit Armature Join
@@ -101,43 +104,33 @@ static void joined_armature_fix_links_constraints(Main *bmain,
         BKE_action_fix_paths_rename(
             &tarArm->id, data->act, "pose.bones[", pchan->name, curbone->name, 0, 0, false);
 
-        DEG_id_tag_update_ex(bmain, &data->act->id, ID_RECALC_COPY_ON_WRITE);
+        DEG_id_tag_update_ex(bmain, &data->act->id, ID_RECALC_SYNC_TO_EVAL);
       }
     }
   }
 
   if (changed) {
-    DEG_id_tag_update_ex(bmain, &ob->id, ID_RECALC_COPY_ON_WRITE);
+    DEG_id_tag_update_ex(bmain, &ob->id, ID_RECALC_SYNC_TO_EVAL);
   }
 }
 
-/** User-data for #joined_armature_fix_animdata_cb(). */
-struct tJoinArmature_AdtFixData {
-  Main *bmain;
-
-  Object *srcArm;
-  Object *tarArm;
-
-  GHash *names_map;
-};
-
 /* Callback to pass to BKE_animdata_main_cb() for fixing driver ID's to point to the new ID. */
 /* FIXME: For now, we only care about drivers here.
  *        When editing rigs, it's very rare to have animation on the rigs being edited already,
  *        so it should be safe to skip these.
  */
-static void joined_armature_fix_animdata_cb(ID *id, FCurve *fcu, void *user_data)
+static void joined_armature_fix_animdata_cb(
+    Main *bmain, ID *id, FCurve *fcu, Object *srcArm, Object *tarArm, GHash *names_map)
 {
-  tJoinArmature_AdtFixData *afd = (tJoinArmature_AdtFixData *)user_data;
-  ID *src_id = &afd->srcArm->id;
-  ID *dst_id = &afd->tarArm->id;
+  ID *src_id = &srcArm->id;
+  ID *dst_id = &tarArm->id;
 
   GHashIterator gh_iter;
   bool changed = false;
 
   /* Fix paths - If this is the target object, it will have some "dirty" paths */
   if ((id == src_id) && strstr(fcu->rna_path, "pose.bones[")) {
-    GHASH_ITER (gh_iter, afd->names_map) {
+    GHASH_ITER (gh_iter, names_map) {
       const char *old_name = static_cast<const char *>(BLI_ghashIterator_getKey(&gh_iter));
       const char *new_name = static_cast<const char *>(BLI_ghashIterator_getValue(&gh_iter));
 
@@ -181,7 +174,7 @@ static void joined_armature_fix_animdata_cb(ID *id, FCurve *fcu, void *user_data
            *      little twists so that we know that it isn't going to clobber the wrong data
            */
           if ((dtar->rna_path && strstr(dtar->rna_path, "pose.bones[")) || (dtar->pchan_name[0])) {
-            GHASH_ITER (gh_iter, afd->names_map) {
+            GHASH_ITER (gh_iter, names_map) {
               const char *old_name = static_cast<const char *>(BLI_ghashIterator_getKey(&gh_iter));
               const char *new_name = static_cast<const char *>(
                   BLI_ghashIterator_getValue(&gh_iter));
@@ -209,7 +202,7 @@ static void joined_armature_fix_animdata_cb(ID *id, FCurve *fcu, void *user_data
   }
 
   if (changed) {
-    DEG_id_tag_update_ex(afd->bmain, id, ID_RECALC_COPY_ON_WRITE);
+    DEG_id_tag_update_ex(bmain, id, ID_RECALC_SYNC_TO_EVAL);
   }
 }
 
@@ -222,7 +215,8 @@ static void joined_armature_fix_links(
 
   /* let's go through all objects in database */
   for (ob = static_cast<Object *>(bmain->objects.first); ob;
-       ob = static_cast<Object *>(ob->id.next)) {
+       ob = static_cast<Object *>(ob->id.next))
+  {
     /* do some object-type specific things */
     if (ob->type == OB_ARMATURE) {
       pose = ob->pose;
@@ -251,11 +245,55 @@ static void joined_armature_fix_links(
       /* make tar armature be new parent */
       ob->parent = tarArm;
 
-      DEG_id_tag_update_ex(bmain, &ob->id, ID_RECALC_COPY_ON_WRITE);
+      DEG_id_tag_update_ex(bmain, &ob->id, ID_RECALC_SYNC_TO_EVAL);
     }
   }
 }
 
+static BoneCollection *join_armature_remap_collection(
+    const bArmature *src_arm,
+    const int src_index,
+    bArmature *dest_arm,
+    blender::Map<std::string, BoneCollection *> &bone_collection_by_name)
+{
+  using namespace blender::animrig;
+  const BoneCollection *bcoll = src_arm->collection_array[src_index];
+
+  /* Check if already remapped. */
+  BoneCollection *mapped = bone_collection_by_name.lookup_default(bcoll->name, nullptr);
+
+  if (mapped) {
+    return mapped;
+  }
+
+  /* Remap the parent collection if necessary. */
+  const int src_parent_index = armature_bonecoll_find_parent_index(src_arm, src_index);
+  int parent_index = -1;
+
+  if (src_parent_index >= 0) {
+    BoneCollection *mapped_parent = join_armature_remap_collection(
+        src_arm, src_parent_index, dest_arm, bone_collection_by_name);
+
+    if (mapped_parent) {
+      parent_index = armature_bonecoll_find_index(dest_arm, mapped_parent);
+    }
+  }
+
+  /* Create the new collection instance. */
+  BoneCollection *new_bcoll = ANIM_armature_bonecoll_new(dest_arm, bcoll->name, parent_index);
+
+  /* Copy collection visibility. */
+  new_bcoll->flags = bcoll->flags;
+
+  /* Copy custom properties. */
+  if (bcoll->prop) {
+    new_bcoll->prop = IDP_CopyProperty_ex(bcoll->prop, 0);
+  }
+
+  bone_collection_by_name.add(bcoll->name, new_bcoll);
+  return new_bcoll;
+}
+
 int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
 {
   Main *bmain = CTX_data_main(C);
@@ -292,12 +330,12 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
 
   /* Inverse transform for all selected armatures in this object,
    * See #object_join_exec for detailed comment on why the safe version is used. */
-  invert_m4_m4_safe_ortho(oimat, ob_active->object_to_world);
+  invert_m4_m4_safe_ortho(oimat, ob_active->object_to_world().ptr());
 
   /* Index bone collections by name.  This is also used later to keep track
    * of collections added from other armatures. */
   blender::Map<std::string, BoneCollection *> bone_collection_by_name;
-  LISTBASE_FOREACH (BoneCollection *, bcoll, &arm->collections) {
+  for (BoneCollection *bcoll : arm->collections_span()) {
     bone_collection_by_name.add(bcoll->name, bcoll);
   }
 
@@ -314,39 +352,23 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
 
   CTX_DATA_BEGIN (C, Object *, ob_iter, selected_editable_objects) {
     if ((ob_iter->type == OB_ARMATURE) && (ob_iter != ob_active)) {
-      tJoinArmature_AdtFixData afd = {nullptr};
       bArmature *curarm = static_cast<bArmature *>(ob_iter->data);
 
       /* we assume that each armature datablock is only used in a single place */
       BLI_assert(ob_active->data != ob_iter->data);
 
       /* init callback data for fixing up AnimData links later */
-      afd.bmain = bmain;
-      afd.srcArm = ob_iter;
-      afd.tarArm = ob_active;
-      afd.names_map = BLI_ghash_str_new("join_armature_adt_fix");
+      GHash *names_map = BLI_ghash_str_new("join_armature_adt_fix");
 
       /* Make a list of edit-bones in current armature */
       ED_armature_to_edit(curarm);
 
-      /* Move new bone collections, and store their remapping info.
-       * TODO: armatures can potentially have multiple users, so these should
-       * actually be copied, not moved.  However, the armature join code is
-       * already broken in that situation.  When that gets fixed, this should
-       * also get fixed.  Note that copying the collections should include
-       * copying their custom properties. (Nathan Vegdahl) */
-      LISTBASE_FOREACH_MUTABLE (BoneCollection *, bcoll, &curarm->collections) {
-        BoneCollection *mapped = bone_collection_by_name.lookup_default(bcoll->name, nullptr);
+      /* Copy new bone collections, and store their remapping info. */
+      for (int i = 0; i < curarm->collection_array_num; i++) {
+        BoneCollection *mapped = join_armature_remap_collection(
+            curarm, i, arm, bone_collection_by_name);
 
-        if (!mapped) {
-          BLI_remlink(&curarm->collections, bcoll);
-          BLI_addtail(&arm->collections, bcoll);
-
-          bone_collection_by_name.add(bcoll->name, bcoll);
-          mapped = bcoll;
-        }
-
-        bone_collection_remap.add(bcoll, mapped);
+        bone_collection_remap.add(curarm->collection_array[i], mapped);
       }
 
       /* Get Pose of current armature */
@@ -355,16 +377,16 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
       // BASACT->flag &= ~OB_MODE_POSE;
 
       /* Find the difference matrix */
-      mul_m4_m4m4(mat, oimat, ob_iter->object_to_world);
+      mul_m4_m4m4(mat, oimat, ob_iter->object_to_world().ptr());
 
-      /* Copy bones and posechannels from the object to the edit armature */
+      /* Copy bones and pose-channels from the object to the edit armature. */
       for (pchan = static_cast<bPoseChannel *>(opose->chanbase.first); pchan; pchan = pchann) {
         pchann = pchan->next;
         curbone = ED_armature_ebone_find_name(curarm->edbo, pchan->name);
 
         /* Get new name */
         ED_armature_ebone_unique_name(arm->edbo, curbone->name, nullptr);
-        BLI_ghash_insert(afd.names_map, BLI_strdup(pchan->name), curbone->name);
+        BLI_ghash_insert(names_map, BLI_strdup(pchan->name), curbone->name);
 
         /* Transform the bone */
         {
@@ -426,8 +448,10 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
       DEG_id_tag_update_ex(bmain, &curarm->id, ID_RECALC_GEOMETRY);
 
       /* Fix all the drivers (and animation data) */
-      BKE_fcurves_main_cb(bmain, joined_armature_fix_animdata_cb, &afd);
-      BLI_ghash_free(afd.names_map, MEM_freeN, nullptr);
+      BKE_fcurves_main_cb(bmain, [&](ID *id, FCurve *fcu) {
+        joined_armature_fix_animdata_cb(bmain, id, fcu, ob_iter, ob_active, names_map);
+      });
+      BLI_ghash_free(names_map, MEM_freeN, nullptr);
 
       /* Only copy over animdata now, after all the remapping has been done,
        * so that we don't have to worry about ambiguities re which armature
@@ -457,7 +481,7 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
       }
 
       /* Free the old object data */
-      ED_object_base_free_and_unlink(bmain, scene, ob_iter);
+      blender::ed::object::base_free_and_unlink(bmain, scene, ob_iter);
     }
   }
   CTX_DATA_END;
@@ -467,6 +491,9 @@ int ED_armature_join_objects_exec(bContext *C, wmOperator *op)
   ED_armature_from_edit(bmain, arm);
   ED_armature_edit_free(arm);
 
+  /* Make sure to recompute bone collection visibility. */
+  ANIM_armature_runtime_refresh(arm);
+
   DEG_id_tag_update(&scene->id, ID_RECALC_SELECT);
   WM_event_add_notifier(C, NC_SCENE | ND_OB_ACTIVE, scene);
   WM_event_add_notifier(C, NC_SCENE | ND_LAYER_CONTENT, scene);
@@ -492,7 +519,8 @@ static void separated_armature_fix_links(Main *bmain, Object *origArm, Object *n
 
   /* let's go through all objects in database */
   for (ob = static_cast<Object *>(bmain->objects.first); ob;
-       ob = static_cast<Object *>(ob->id.next)) {
+       ob = static_cast<Object *>(ob->id.next))
+  {
     /* do some object-type specific things */
     if (ob->type == OB_ARMATURE) {
       LISTBASE_FOREACH (bPoseChannel *, pchan, &ob->pose->chanbase) {
@@ -644,12 +672,10 @@ static int separate_armature_exec(bContext *C, wmOperator *op)
   /* set wait cursor in case this takes a while */
   WM_cursor_wait(true);
 
-  uint bases_len = 0;
-  Base **bases = BKE_view_layer_array_from_bases_in_edit_mode_unique_data(
-      scene, view_layer, CTX_wm_view3d(C), &bases_len);
+  Vector<Base *> bases = BKE_view_layer_array_from_bases_in_edit_mode_unique_data(
+      scene, view_layer, CTX_wm_view3d(C));
 
-  for (uint base_index = 0; base_index < bases_len; base_index++) {
-    Base *base_old = bases[base_index];
+  for (Base *base_old : bases) {
     Object *ob_old = base_old->object;
 
     {
@@ -697,7 +723,7 @@ static int separate_armature_exec(bContext *C, wmOperator *op)
     /* Only duplicate linked armature but take into account
      * user preferences for duplicating actions. */
     short dupflag = USER_DUP_ARM | (U.dupflag & USER_DUP_ACT);
-    Base *base_new = ED_object_add_duplicate(
+    Base *base_new = blender::ed::object::add_duplicate(
         bmain, scene, view_layer, base_old, eDupli_ID_Flags(dupflag));
     Object *ob_new = base_new->object;
 
@@ -724,7 +750,6 @@ static int separate_armature_exec(bContext *C, wmOperator *op)
     /* NOTE: notifier might evolve. */
     WM_event_add_notifier(C, NC_OBJECT | ND_POSE, ob_old);
   }
-  MEM_freeN(bases);
 
   /* Recalculate/redraw + cleanup */
   WM_cursor_wait(false);
@@ -745,13 +770,11 @@ void ARMATURE_OT_separate(wmOperatorType *ot)
   ot->description = "Isolate selected bones into a separate armature";
 
   /* callbacks */
-  ot->invoke = WM_operator_confirm_or_exec;
   ot->exec = separate_armature_exec;
   ot->poll = ED_operator_editarmature;
 
   /* flags */
   ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-  WM_operator_properties_confirm_or_exec(ot);
 }
 
 /** \} */
@@ -867,7 +890,7 @@ static int armature_parent_set_exec(bContext *C, wmOperator *op)
   if (actbone->flag & BONE_SELECTED) {
     is_active_only_selected = true;
     LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
-      if (EBONE_EDITABLE(ebone) && (ebone->flag & BONE_SELECTED)) {
+      if (EBONE_EDITABLE(ebone)) {
         if (ebone != actbone) {
           is_active_only_selected = false;
           break;
@@ -899,7 +922,7 @@ static int armature_parent_set_exec(bContext *C, wmOperator *op)
 
     /* Parent selected bones to the active one. */
     LISTBASE_FOREACH (EditBone *, ebone, arm->edbo) {
-      if (EBONE_EDITABLE(ebone) && (ebone->flag & BONE_SELECTED)) {
+      if (EBONE_EDITABLE(ebone)) {
         if (ebone != actbone) {
           bone_connect_to_new_parent(arm->edbo, ebone, actbone, val);
         }
@@ -958,11 +981,13 @@ static int armature_parent_set_invoke(bContext *C, wmOperator * /*op*/, const wm
 
   uiLayout *row_offset = uiLayoutRow(layout, false);
   uiLayoutSetEnabled(row_offset, enable_offset);
-  uiItemEnumO(row_offset, "ARMATURE_OT_parent_set", nullptr, ICON_NONE, "type", ARM_PAR_OFFSET);
+  uiItemEnumO(
+      row_offset, "ARMATURE_OT_parent_set", std::nullopt, ICON_NONE, "type", ARM_PAR_OFFSET);
 
   uiLayout *row_connect = uiLayoutRow(layout, false);
   uiLayoutSetEnabled(row_connect, enable_connect);
-  uiItemEnumO(row_connect, "ARMATURE_OT_parent_set", nullptr, ICON_NONE, "type", ARM_PAR_CONNECT);
+  uiItemEnumO(
+      row_connect, "ARMATURE_OT_parent_set", std::nullopt, ICON_NONE, "type", ARM_PAR_CONNECT);
 
   UI_popup_menu_end(C, pup);
 
@@ -1018,11 +1043,9 @@ static int armature_parent_clear_exec(bContext *C, wmOperator *op)
   }
   CTX_DATA_END;
 
-  uint objects_len = 0;
-  Object **objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data(
-      scene, view_layer, CTX_wm_view3d(C), &objects_len);
-  for (uint ob_index = 0; ob_index < objects_len; ob_index++) {
-    Object *ob = objects[ob_index];
+  Vector<Object *> objects = BKE_view_layer_array_from_objects_in_edit_mode_unique_data(
+      scene, view_layer, CTX_wm_view3d(C));
+  for (Object *ob : objects) {
     bArmature *arm = static_cast<bArmature *>(ob->data);
     bool changed = false;
 
@@ -1042,8 +1065,6 @@ static int armature_parent_clear_exec(bContext *C, wmOperator *op)
     /* NOTE: notifier might evolve. */
     WM_event_add_notifier(C, NC_OBJECT | ND_BONE_SELECT, ob);
   }
-  MEM_freeN(objects);
-
   return OPERATOR_FINISHED;
 }
 
@@ -1080,13 +1101,14 @@ static int armature_parent_clear_invoke(bContext *C,
 
   uiLayout *row_clear = uiLayoutRow(layout, false);
   uiLayoutSetEnabled(row_clear, enable_clear);
-  uiItemEnumO(row_clear, "ARMATURE_OT_parent_clear", nullptr, ICON_NONE, "type", ARM_PAR_CLEAR);
+  uiItemEnumO(
+      row_clear, "ARMATURE_OT_parent_clear", std::nullopt, ICON_NONE, "type", ARM_PAR_CLEAR);
 
   uiLayout *row_disconnect = uiLayoutRow(layout, false);
   uiLayoutSetEnabled(row_disconnect, enable_disconnect);
   uiItemEnumO(row_disconnect,
               "ARMATURE_OT_parent_clear",
-              nullptr,
+              std::nullopt,
               ICON_NONE,
               "type",
               ARM_PAR_CLEAR_DISCONNECT);
