@@ -6,6 +6,7 @@
  * \ingroup ikplugin
  */
 
+#include <algorithm>
 #include <cmath>
 #include <cstdlib>
 #include <cstring>
@@ -26,16 +27,13 @@
 #include "MEM_guardedalloc.h"
 
 #include "BIK_api.h"
-#include "BLI_blenlib.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
 #include "BLI_math_vector.h"
-#include "BLI_utildefines.h"
 
-#include "BKE_action.h"
-#include "BKE_armature.h"
+#include "BKE_action.hh"
+#include "BKE_armature.hh"
 #include "BKE_constraint.h"
-#include "BKE_global.h"
 #include "DNA_action_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_constraint_types.h"
@@ -212,7 +210,7 @@ enum IK_SegmentAxis {
   IK_TRANS_Z = 5,
 };
 
-static int initialize_chain(Object *ob, bPoseChannel *pchan_tip, bConstraint *con)
+static int initialize_chain(Object * /*ob*/, bPoseChannel *pchan_tip, bConstraint *con)
 {
   bPoseChannel *curchan, *pchan_root = nullptr, *chanlist[256], **oldchan;
   PoseTree *tree;
@@ -299,11 +297,11 @@ static int initialize_chain(Object *ob, bPoseChannel *pchan_tip, bConstraint *co
     treecount = 1;
   }
   else {
-    tree->iterations = MAX2(data->iterations, tree->iterations);
+    tree->iterations = std::max<int>(data->iterations, tree->iterations);
     tree->stretch = tree->stretch && !(data->flag & CONSTRAINT_IK_STRETCH);
 
     /* Skip common pose channels and add remaining. */
-    size = MIN2(segcount, tree->totchannel);
+    size = std::min(segcount, tree->totchannel);
     a = t = 0;
     while (a < size && t < tree->totchannel) {
       /* locate first matching channel */
@@ -314,7 +312,8 @@ static int initialize_chain(Object *ob, bPoseChannel *pchan_tip, bConstraint *co
         break;
       }
       for (; a < size && t < tree->totchannel && tree->pchan[t] == chanlist[segcount - a - 1];
-           a++, t++) {
+           a++, t++)
+      {
         /* pass */
       }
     }
@@ -366,7 +365,7 @@ static int initialize_chain(Object *ob, bPoseChannel *pchan_tip, bConstraint *co
   return treecount;
 }
 
-static bool is_cartesian_constraint(bConstraint *con)
+static bool is_cartesian_constraint(bConstraint * /*con*/)
 {
   // bKinematicConstraint* data=(bKinematicConstraint *)con->data;
 
@@ -554,15 +553,15 @@ static void GetJointRotation(KDL::Rotation &boneRot, int type, double *rot)
   }
 }
 
-static bool target_callback(const iTaSC::Timestamp &timestamp,
-                            const iTaSC::Frame &current,
+static bool target_callback(const iTaSC::Timestamp & /*timestamp*/,
+                            const iTaSC::Frame & /*current*/,
                             iTaSC::Frame &next,
                             void *param)
 {
   IK_Target *target = (IK_Target *)param;
   /* compute next target position
    * get target matrix from constraint. */
-  bConstraint *constraint = (bConstraint *)target->blenderConstraint;
+  bConstraint *constraint = target->blenderConstraint;
   float tarmat[4][4];
 
   BKE_constraint_target_matrix_get(target->bldepsgraph,
@@ -587,10 +586,10 @@ static bool target_callback(const iTaSC::Timestamp &timestamp,
       float chanmat[4][4];
       copy_m4_m4(chanmat, pchan->pose_mat);
       copy_v3_v3(chanmat[3], pchan->pose_tail);
-      mul_m4_series(restmat, target->owner->object_to_world, chanmat, target->eeRest);
+      mul_m4_series(restmat, target->owner->object_to_world().ptr(), chanmat, target->eeRest);
     }
     else {
-      mul_m4_m4m4(restmat, target->owner->object_to_world, target->eeRest);
+      mul_m4_m4m4(restmat, target->owner->object_to_world().ptr(), target->eeRest);
     }
     /* blend the target */
     blend_m4_m4m4(tarmat, restmat, tarmat, constraint->enforce);
@@ -600,7 +599,7 @@ static bool target_callback(const iTaSC::Timestamp &timestamp,
 }
 
 static bool base_callback(const iTaSC::Timestamp &timestamp,
-                          const iTaSC::Frame &current,
+                          const iTaSC::Frame & /*current*/,
                           iTaSC::Frame &next,
                           void *param)
 {
@@ -621,10 +620,10 @@ static bool base_callback(const iTaSC::Timestamp &timestamp,
     ikscene->baseFrame.setValue(&chanmat[0][0]);
     /* iTaSC armature is scaled to object scale, scale the base frame too */
     ikscene->baseFrame.p *= ikscene->blScale;
-    mul_m4_m4m4(rootmat, ikscene->blArmature->object_to_world, chanmat);
+    mul_m4_m4m4(rootmat, ikscene->blArmature->object_to_world().ptr(), chanmat);
   }
   else {
-    copy_m4_m4(rootmat, ikscene->blArmature->object_to_world);
+    copy_m4_m4(rootmat, ikscene->blArmature->object_to_world().ptr());
     ikscene->baseFrame = iTaSC::F_identity;
   }
   next.setValue(&rootmat[0][0]);
@@ -703,9 +702,9 @@ static bool base_callback(const iTaSC::Timestamp &timestamp,
   return true;
 }
 
-static bool copypose_callback(const iTaSC::Timestamp &timestamp,
+static bool copypose_callback(const iTaSC::Timestamp & /*timestamp*/,
                               iTaSC::ConstraintValues *const _values,
-                              uint _nvalues,
+                              uint /*nvalues*/,
                               void *_param)
 {
   IK_Target *iktarget = (IK_Target *)_param;
@@ -750,7 +749,7 @@ static bool copypose_callback(const iTaSC::Timestamp &timestamp,
 }
 
 static void copypose_error(const iTaSC::ConstraintValues *values,
-                           uint nvalues,
+                           uint /*nvalues*/,
                            IK_Target *iktarget)
 {
   iTaSC::ConstraintSingleValue *value;
@@ -777,7 +776,7 @@ static void copypose_error(const iTaSC::ConstraintValues *values,
 
 static bool distance_callback(const iTaSC::Timestamp &timestamp,
                               iTaSC::ConstraintValues *const _values,
-                              uint _nvalues,
+                              uint /*nvalues*/,
                               void *_param)
 {
   IK_Target *iktarget = (IK_Target *)_param;
@@ -827,13 +826,13 @@ static bool distance_callback(const iTaSC::Timestamp &timestamp,
 }
 
 static void distance_error(const iTaSC::ConstraintValues *values,
-                           uint _nvalues,
+                           uint /*nvalues*/,
                            IK_Target *iktarget)
 {
   iktarget->blenderConstraint->lin_error = float(values->values[0].y - values->values[0].yd);
 }
 
-static bool joint_callback(const iTaSC::Timestamp &timestamp,
+static bool joint_callback(const iTaSC::Timestamp & /*timestamp*/,
                            iTaSC::ConstraintValues *const _values,
                            uint _nvalues,
                            void *_param)
@@ -1066,7 +1065,7 @@ static void convert_pose(IK_Scene *ikscene)
   int a, joint;
 
   /* assume uniform scaling and take Y scale as general scale for the armature */
-  scale = len_v3(ikscene->blArmature->object_to_world[1]);
+  scale = len_v3(ikscene->blArmature->object_to_world().ptr()[1]);
   rot = ikscene->jointArray(0);
   for (joint = a = 0, ikchan = ikscene->channels;
        a < ikscene->numchan && joint < ikscene->numjoint;
@@ -1107,7 +1106,7 @@ static void BKE_pose_rest(IK_Scene *ikscene)
   int a, joint;
 
   /* assume uniform scaling and take Y scale as general scale for the armature */
-  scale = len_v3(ikscene->blArmature->object_to_world[1]);
+  scale = len_v3(ikscene->blArmature->object_to_world().ptr()[1]);
   /* rest pose is 0 */
   SetToZero(ikscene->jointArray);
   /* except for transY joints */
@@ -1186,7 +1185,7 @@ static IK_Scene *convert_tree(
   }
   ikscene->blArmature = ob;
   /* assume uniform scaling and take Y scale as general scale for the armature */
-  ikscene->blScale = len_v3(ob->object_to_world[1]);
+  ikscene->blScale = len_v3(ob->object_to_world().ptr()[1]);
   ikscene->blInvScale = (ikscene->blScale < KDL::epsilon) ? 0.0f : 1.0f / ikscene->blScale;
 
   std::string joint;
@@ -1433,7 +1432,7 @@ static IK_Scene *convert_tree(
   /* for each target, we need to add an end effector in the armature */
   for (numtarget = 0, polarcon = nullptr, ret = true, target = (PoseTarget *)tree->targets.first;
        target;
-       target = (PoseTarget *)target->next)
+       target = target->next)
   {
     condata = (bKinematicConstraint *)target->con->data;
     pchan = tree->pchan[target->tip];
@@ -1674,7 +1673,7 @@ static void create_scene(Depsgraph *depsgraph, Scene *scene, Object *ob, float c
 static int init_scene(Object *ob)
 {
   /* check also if scaling has changed */
-  float scale = len_v3(ob->object_to_world[1]);
+  float scale = len_v3(ob->object_to_world().ptr()[1]);
   IK_Scene *scene;
 
   if (ob->pose->ikdata) {
@@ -1920,7 +1919,7 @@ void itasc_execute_tree(
   }
 }
 
-void itasc_release_tree(Scene *scene, Object *ob, float ctime)
+void itasc_release_tree(Scene * /*scene*/, Object * /*ob*/, float /*ctime*/)
 {
   /* not used for iTaSC */
 }
@@ -1980,7 +1979,7 @@ void itasc_update_param(bPose *pose)
   }
 }
 
-void itasc_test_constraint(Object *ob, bConstraint *cons)
+void itasc_test_constraint(Object * /*ob*/, bConstraint *cons)
 {
   bKinematicConstraint *data = (bKinematicConstraint *)cons->data;
 
