@@ -8,12 +8,10 @@
 
 #include <cstddef>
 
-#include "BLI_utildefines.h"
-
-#include "IMB_allocimbuf.h"
-#include "IMB_colormanagement_intern.h"
-#include "IMB_filetype.h"
-#include "IMB_imbuf.h"
+#include "IMB_allocimbuf.hh"
+#include "IMB_colormanagement_intern.hh"
+#include "IMB_filetype.hh"
+#include "IMB_imbuf.hh"
 
 void IMB_init()
 {
