@@ -11,12 +11,10 @@
 #include "DNA_scene_types.h"
 #include "DNA_sequence_types.h"
 
-#include "BKE_scene.h"
+#include "SEQ_select.hh"
+#include "SEQ_sequencer.hh"
 
-#include "SEQ_select.h"
-#include "SEQ_sequencer.h"
-
-Sequence *SEQ_select_active_get(Scene *scene)
+Strip *SEQ_select_active_get(const Scene *scene)
 {
   const Editing *ed = SEQ_editing_get(scene);
 
@@ -27,7 +25,7 @@ Sequence *SEQ_select_active_get(Scene *scene)
   return ed->act_seq;
 }
 
-void SEQ_select_active_set(Scene *scene, Sequence *seq)
+void SEQ_select_active_set(Scene *scene, Strip *strip)
 {
   Editing *ed = SEQ_editing_get(scene);
 
@@ -35,10 +33,10 @@ void SEQ_select_active_set(Scene *scene, Sequence *seq)
     return;
   }
 
-  ed->act_seq = seq;
+  ed->act_seq = strip;
 }
 
-bool SEQ_select_active_get_pair(Scene *scene, Sequence **r_seq_act, Sequence **r_seq_other)
+bool SEQ_select_active_get_pair(Scene *scene, Strip **r_seq_act, Strip **r_seq_other)
 {
   Editing *ed = SEQ_editing_get(scene);
 
@@ -50,13 +48,13 @@ bool SEQ_select_active_get_pair(Scene *scene, Sequence **r_seq_act, Sequence **r
 
   *r_seq_other = nullptr;
 
-  LISTBASE_FOREACH (Sequence *, seq, ed->seqbasep) {
-    if (seq->flag & SELECT && (seq != (*r_seq_act))) {
+  LISTBASE_FOREACH (Strip *, strip, ed->seqbasep) {
+    if (strip->flag & SELECT && (strip != (*r_seq_act))) {
       if (*r_seq_other) {
         return false;
       }
 
-      *r_seq_other = seq;
+      *r_seq_other = strip;
     }
   }
 
