@@ -19,35 +19,35 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
-#include "BLI_bitmap.h"
-#include "BLI_blenlib.h"
 #include "BLI_math_color.h"
+#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 
 #include "BIF_glutil.hh"
 
-#include "BKE_context.h"
-#include "BKE_global.h"
-#include "BKE_idprop.h"
-#include "BKE_idtype.h"
-#include "BKE_lib_id.h"
-#include "BKE_main.h"
+#include "BKE_context.hh"
+#include "BKE_global.hh"
+#include "BKE_idprop.hh"
+#include "BKE_idtype.hh"
+#include "BKE_lib_id.hh"
+#include "BKE_main.hh"
+#include "BKE_preview_image.hh"
 #include "BKE_screen.hh"
 
-#include "GHOST_C-api.h"
+#include "BLO_readfile.hh"
 
-#include "BLO_readfile.h"
-
-#include "ED_asset.hh"
 #include "ED_fileselect.hh"
 #include "ED_screen.hh"
 
-#include "GPU_shader.h"
-#include "GPU_state.h"
-#include "GPU_viewport.h"
+#include "GPU_shader_builtin.hh"
+#include "GPU_state.hh"
 
-#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
+
+#include "GHOST_Types.h"
 
 #include "UI_interface.hh"
 #include "UI_interface_icons.hh"
@@ -57,9 +57,10 @@
 
 #include "WM_api.hh"
 #include "WM_types.hh"
-#include "wm_event_system.h"
+#include "wm_event_system.hh"
 #include "wm_window.hh"
 
+#include <fmt/format.h>
 /* ****************************************************** */
 
 static ListBase dropboxes = {nullptr, nullptr};
@@ -67,9 +68,15 @@ static ListBase dropboxes = {nullptr, nullptr};
 static void wm_drag_free_asset_data(wmDragAsset **asset_data);
 static void wm_drag_free_path_data(wmDragPath **path_data);
 
-/* drop box maps are stored global for now */
-/* these are part of blender's UI/space specs, and not like keymaps */
-/* when editors become configurable, they can add own dropbox definitions */
+static void wm_drop_item_free_data(wmDropBox *drop);
+static void wm_drop_item_clear_runtime(wmDropBox *drop);
+
+wmDragActiveDropState::wmDragActiveDropState() = default;
+wmDragActiveDropState::~wmDragActiveDropState() = default;
+
+/* Drop box maps are stored global for now. */
+/* These are part of blender's UI/space specs, and not like keymaps. */
+/* When editors become configurable, they can add their own dropbox definitions. */
 
 struct wmDropBoxMap {
   wmDropBoxMap *next, *prev;
@@ -100,39 +107,110 @@ ListBase *WM_dropboxmap_find(const char *idname, int spaceid, int regionid)
 
 wmDropBox *WM_dropbox_add(ListBase *lb,
                           const char *idname,
-                          bool (*poll)(bContext *, wmDrag *, const wmEvent *),
-                          void (*copy)(bContext *, wmDrag *, wmDropBox *),
-                          void (*cancel)(Main *, wmDrag *, wmDropBox *),
+                          bool (*poll)(bContext *C, wmDrag *drag, const wmEvent *event),
+                          void (*copy)(bContext *C, wmDrag *drag, wmDropBox *drop),
+                          void (*cancel)(Main *bmain, wmDrag *drag, wmDropBox *drop),
                           WMDropboxTooltipFunc tooltip)
 {
+  wmOperatorType *ot = WM_operatortype_find(idname, true);
+  if (ot == nullptr) {
+    printf("Error: dropbox with unknown operator: %s\n", idname);
+    return nullptr;
+  }
+
   wmDropBox *drop = MEM_cnew<wmDropBox>(__func__);
   drop->poll = poll;
   drop->copy = copy;
   drop->cancel = cancel;
   drop->tooltip = tooltip;
-  drop->ot = WM_operatortype_find(idname, false);
+  drop->ot = ot;
+  STRNCPY(drop->opname, idname);
 
-  if (drop->ot == nullptr) {
-    MEM_freeN(drop);
-    printf("Error: dropbox with unknown operator: %s\n", idname);
-    return nullptr;
-  }
   WM_operator_properties_alloc(&(drop->ptr), &(drop->properties), idname);
+  WM_operator_properties_sanitize(drop->ptr, true);
+
+  /* Signal for no context, see #STRUCT_NO_CONTEXT_WITHOUT_OWNER_ID. */
+  drop->ptr->owner_id = nullptr;
 
   BLI_addtail(lb, drop);
 
   return drop;
 }
 
+static void wm_dropbox_item_update_ot(wmDropBox *drop)
+{
+  /* NOTE(@ideasman42): this closely follows #wm_keymap_item_properties_update_ot.
+   * `keep_properties` is implied because drop boxes aren't dynamically added & removed.
+   * It's possible in the future drop-boxes can be (un)registered by scripts.
+   * In this case we might want to remove drop-boxes that point to missing operators. */
+  wmOperatorType *ot = WM_operatortype_find(drop->opname, false);
+  if (ot == nullptr) {
+    /* Allow for the operator to be added back and re-validated, keep it's properties. */
+    wm_drop_item_clear_runtime(drop);
+    drop->ot = nullptr;
+    return;
+  }
+
+  if (drop->ptr == nullptr) {
+    WM_operator_properties_alloc(&(drop->ptr), &(drop->properties), drop->opname);
+    WM_operator_properties_sanitize(drop->ptr, true);
+  }
+  else {
+    if (ot->srna != drop->ptr->type) {
+      WM_operator_properties_create_ptr(drop->ptr, ot);
+      if (drop->properties) {
+        drop->ptr->data = drop->properties;
+      }
+      WM_operator_properties_sanitize(drop->ptr, true);
+    }
+  }
+
+  if (drop->ptr) {
+    drop->ptr->owner_id = nullptr;
+  }
+  drop->ot = ot;
+}
+
+void WM_dropbox_update_ot()
+{
+  LISTBASE_FOREACH (wmDropBoxMap *, dm, &dropboxes) {
+    LISTBASE_FOREACH (wmDropBox *, drop, &dm->dropboxes) {
+      wm_dropbox_item_update_ot(drop);
+    }
+  }
+}
+
+static void wm_drop_item_free_data(wmDropBox *drop)
+{
+  if (drop->ptr) {
+    WM_operator_properties_free(drop->ptr);
+    MEM_delete(drop->ptr);
+    drop->ptr = nullptr;
+    drop->properties = nullptr;
+  }
+  else if (drop->properties) {
+    IDP_FreeProperty(drop->properties);
+    drop->properties = nullptr;
+  }
+}
+
+static void wm_drop_item_clear_runtime(wmDropBox *drop)
+{
+  IDProperty *properties = drop->properties;
+  drop->properties = nullptr;
+  if (drop->ptr) {
+    drop->ptr->data = nullptr;
+  }
+  wm_drop_item_free_data(drop);
+  drop->properties = properties;
+}
+
 void wm_dropbox_free()
 {
 
   LISTBASE_FOREACH (wmDropBoxMap *, dm, &dropboxes) {
     LISTBASE_FOREACH (wmDropBox *, drop, &dm->dropboxes) {
-      if (drop->ptr) {
-        WM_operator_properties_free(drop->ptr);
-        MEM_freeN(drop->ptr);
-      }
+      wm_drop_item_free_data(drop);
     }
     BLI_freelistN(&dm->dropboxes);
   }
@@ -154,7 +232,7 @@ static void wm_dropbox_invoke(bContext *C, wmDrag *drag)
     bScreen *screen = WM_window_get_active_screen(win);
     ED_screen_areas_iter (win, screen, area) {
       LISTBASE_FOREACH (ARegion *, region, &area->regionbase) {
-        if (region->visible) {
+        if (region->runtime->visible) {
           BLI_assert(area->spacetype < SPACE_TYPE_NUM);
           BLI_assert(region->regiontype < RGN_TYPE_NUM);
           area_region_tag[area->spacetype][region->regiontype] = true;
@@ -180,13 +258,12 @@ static void wm_dropbox_invoke(bContext *C, wmDrag *drag)
   }
 }
 
-wmDrag *WM_drag_data_create(
-    bContext *C, int icon, eWM_DragDataType type, void *poin, double value, uint flags)
+wmDrag *WM_drag_data_create(bContext *C, int icon, eWM_DragDataType type, void *poin, uint flags)
 {
   wmDrag *drag = MEM_new<wmDrag>(__func__);
 
   /* Keep track of future multi-touch drag too, add a mouse-pointer id or so. */
-  /* if multiple drags are added, they're drawn as list */
+  /* If multiple drags are added, they're drawn as list. */
 
   drag->flags = static_cast<eWM_DragFlags>(flags);
   drag->icon = icon;
@@ -211,20 +288,18 @@ wmDrag *WM_drag_data_create(
       /* The asset-list case is special: We get multiple assets from context and attach them to the
        * drag item. */
     case WM_DRAG_ASSET_LIST: {
-      ListBase asset_links = CTX_data_collection_get(C, "selected_assets");
-      LISTBASE_FOREACH (const CollectionPointerLink *, link, &asset_links) {
+      blender::Vector<PointerRNA> asset_links = CTX_data_collection_get(C, "selected_assets");
+      for (const PointerRNA &ptr : asset_links) {
         const AssetRepresentationHandle *asset = static_cast<const AssetRepresentationHandle *>(
-            link->ptr.data);
+            ptr.data);
         WM_drag_add_asset_list_item(drag, asset);
       }
-      BLI_freelistN(&asset_links);
       break;
     }
     default:
       drag->poin = poin;
       break;
   }
-  drag->value = value;
 
   return drag;
 }
@@ -237,10 +312,9 @@ void WM_event_start_prepared_drag(bContext *C, wmDrag *drag)
   wm_dropbox_invoke(C, drag);
 }
 
-void WM_event_start_drag(
-    bContext *C, int icon, eWM_DragDataType type, void *poin, double value, uint flags)
+void WM_event_start_drag(bContext *C, int icon, eWM_DragDataType type, void *poin, uint flags)
 {
-  wmDrag *drag = WM_drag_data_create(C, icon, type, poin, value, flags);
+  wmDrag *drag = WM_drag_data_create(C, icon, type, poin, flags);
   WM_event_start_prepared_drag(C, drag);
 }
 
@@ -251,7 +325,7 @@ void wm_drags_exit(wmWindowManager *wm, wmWindow *win)
     WM_cursor_modal_restore(win);
   }
 
-  /* Active area should always redraw, even if cancelled. */
+  /* Active area should always redraw, even if canceled. */
   int event_xy_target[2];
   wmWindow *target_win = WM_window_find_under_cursor(win, win->eventstate->xy, event_xy_target);
   if (target_win) {
@@ -285,6 +359,12 @@ void WM_event_drag_image(wmDrag *drag, const ImBuf *imb, float scale)
   drag->imbuf_scale = scale;
 }
 
+void WM_event_drag_preview_icon(wmDrag *drag, int icon_id)
+{
+  BLI_assert_msg(!drag->imb, "Drag image and preview are mutually exclusive");
+  drag->preview_icon_id = icon_id;
+}
+
 void WM_drag_data_free(eWM_DragDataType dragtype, void *poin)
 {
   /* Don't require all the callers to have a nullptr-check, just allow passing nullptr. */
@@ -304,6 +384,11 @@ void WM_drag_data_free(eWM_DragDataType dragtype, void *poin)
       wm_drag_free_path_data(&path_data);
       break;
     }
+    case WM_DRAG_STRING: {
+      std::string *str = static_cast<std::string *>(poin);
+      MEM_delete(str);
+      break;
+    }
     default:
       MEM_freeN(poin);
       break;
@@ -312,8 +397,8 @@ void WM_drag_data_free(eWM_DragDataType dragtype, void *poin)
 
 void WM_drag_free(wmDrag *drag)
 {
-  if (drag->drop_state.active_dropbox && drag->drop_state.active_dropbox->draw_deactivate) {
-    drag->drop_state.active_dropbox->draw_deactivate(drag->drop_state.active_dropbox, drag);
+  if (drag->drop_state.active_dropbox && drag->drop_state.active_dropbox->on_exit) {
+    drag->drop_state.active_dropbox->on_exit(drag->drop_state.active_dropbox, drag);
   }
   if (drag->flags & WM_DRAG_FREE_DATA) {
     WM_drag_data_free(drag->type, drag->poin);
@@ -339,18 +424,15 @@ void WM_drag_free_list(ListBase *lb)
   }
 }
 
-static char *dropbox_tooltip(bContext *C, wmDrag *drag, const int xy[2], wmDropBox *drop)
+static std::string dropbox_tooltip(bContext *C, wmDrag *drag, const int xy[2], wmDropBox *drop)
 {
-  char *tooltip = nullptr;
   if (drop->tooltip) {
-    tooltip = drop->tooltip(C, drag, xy, drop);
+    return drop->tooltip(C, drag, xy, drop);
   }
-  if (!tooltip) {
-    tooltip = BLI_strdup(WM_operatortype_name(drop->ot, drop->ptr).c_str());
+  if (drop->ot) {
+    return WM_operatortype_name(drop->ot, drop->ptr);
   }
-  /* XXX Doing translation here might not be ideal, but later we have no more
-   *     access to ot (and hence op context)... */
-  return tooltip;
+  return {};
 }
 
 static wmDropBox *dropbox_active(bContext *C,
@@ -380,7 +462,9 @@ static wmDropBox *dropbox_active(bContext *C,
           }
 
           const wmOperatorCallContext opcontext = wm_drop_operator_context_get(drop);
-          if (WM_operator_poll_context(C, drop->ot, opcontext)) {
+          if (drop->ot && WM_operator_poll_context(C, drop->ot, opcontext)) {
+            /* Get dropbox tooltip now, #wm_drag_draw_tooltip can use a different draw context. */
+            drag->drop_state.tooltip = dropbox_tooltip(C, drag, event->xy, drop);
             CTX_store_set(C, nullptr);
             return drop;
           }
@@ -401,7 +485,7 @@ static wmDropBox *dropbox_active(bContext *C,
   return nullptr;
 }
 
-/* return active operator tooltip/name when mouse is in box */
+/* Return active operator tooltip/name when mouse is in box. */
 static wmDropBox *wm_dropbox_active(bContext *C, wmDrag *drag, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
@@ -412,7 +496,7 @@ static wmDropBox *wm_dropbox_active(bContext *C, wmDrag *drag, const wmEvent *ev
   if (area) {
     ARegion *region = BKE_area_find_region_xy(area, RGN_TYPE_ANY, event->xy);
     if (region) {
-      drop = dropbox_active(C, &region->handlers, drag, event);
+      drop = dropbox_active(C, &region->runtime->handlers, drag, event);
     }
 
     if (!drop) {
@@ -431,11 +515,11 @@ static wmDropBox *wm_dropbox_active(bContext *C, wmDrag *drag, const wmEvent *ev
 static void wm_drop_update_active(bContext *C, wmDrag *drag, const wmEvent *event)
 {
   wmWindow *win = CTX_wm_window(C);
-  const int winsize_x = WM_window_pixels_x(win);
-  const int winsize_y = WM_window_pixels_y(win);
+  const blender::int2 win_size = WM_window_native_pixel_size(win);
 
   /* For multi-window drags, we only do this if mouse inside. */
-  if (event->xy[0] < 0 || event->xy[1] < 0 || event->xy[0] > winsize_x || event->xy[1] > winsize_y)
+  if (event->xy[0] < 0 || event->xy[1] < 0 || event->xy[0] > win_size[0] ||
+      event->xy[1] > win_size[1])
   {
     return;
   }
@@ -443,16 +527,17 @@ static void wm_drop_update_active(bContext *C, wmDrag *drag, const wmEvent *even
   /* Update UI context, before polling so polls can query this context. */
   drag->drop_state.ui_context.reset();
   drag->drop_state.ui_context = wm_drop_ui_context_create(C);
+  drag->drop_state.tooltip = "";
 
   wmDropBox *drop_prev = drag->drop_state.active_dropbox;
   wmDropBox *drop = wm_dropbox_active(C, drag, event);
   if (drop != drop_prev) {
-    if (drop_prev && drop_prev->draw_deactivate) {
-      drop_prev->draw_deactivate(drop_prev, drag);
+    if (drop_prev && drop_prev->on_exit) {
+      drop_prev->on_exit(drop_prev, drag);
       BLI_assert(drop_prev->draw_data == nullptr);
     }
-    if (drop && drop->draw_activate) {
-      drop->draw_activate(drop, drag);
+    if (drop && drop->on_enter) {
+      drop->on_enter(drop, drag);
     }
     drag->drop_state.active_dropbox = drop;
     drag->drop_state.area_from = drop ? CTX_wm_area(C) : nullptr;
@@ -580,7 +665,10 @@ wmDragAsset *WM_drag_create_asset_data(const blender::asset_system::AssetReprese
 
 static void wm_drag_free_asset_data(wmDragAsset **asset_data)
 {
-  MEM_SAFE_FREE(*asset_data);
+  if (*asset_data) {
+    MEM_delete(*asset_data);
+    *asset_data = nullptr;
+  }
 }
 
 wmDragAsset *WM_drag_get_asset_data(const wmDrag *drag, int idcode)
@@ -617,7 +705,7 @@ ID *WM_drag_asset_id_import(const bContext *C, wmDragAsset *asset_drag, const in
                               FILE_ACTIVE_COLLECTION;
 
   const char *name = asset_drag->asset->get_name().c_str();
-  const std::string blend_path = asset_drag->asset->get_identifier().full_library_path();
+  const std::string blend_path = asset_drag->asset->full_library_path();
   const ID_Type idtype = asset_drag->asset->get_id_type();
   const bool use_relative_path = asset_drag->asset->get_use_relative_path();
 
@@ -707,9 +795,9 @@ void WM_drag_free_imported_drag_ID(Main *bmain, wmDrag *drag, wmDropBox *drop)
   }
 
   ID_Type asset_id_type = asset_drag->asset->get_id_type();
-  /* Try to find the imported ID. For this to work either a "session_uuid" or "name" property must
+  /* Try to find the imported ID. For this to work either a "session_uid" or "name" property must
    * have been defined (see #WM_operator_properties_id_lookup()). */
-  ID *id = WM_operator_properties_id_lookup_from_name_or_session_uuid(
+  ID *id = WM_operator_properties_id_lookup_from_name_or_session_uid(
       bmain, drop->ptr, asset_id_type);
   if (id != nullptr) {
     /* Do not delete the dragged ID if it has any user, otherwise if it is a 're-used' ID it will
@@ -759,29 +847,80 @@ const ListBase *WM_drag_asset_list_get(const wmDrag *drag)
   return &drag->asset_items;
 }
 
-wmDragPath *WM_drag_create_path_data(const char *path)
+wmDragPath *WM_drag_create_path_data(blender::Span<const char *> paths)
 {
+  BLI_assert(!paths.is_empty());
   wmDragPath *path_data = MEM_new<wmDragPath>("wmDragPath");
-  path_data->path = BLI_strdup(path);
-  path_data->file_type = ED_path_extension_type(path);
+
+  for (const char *path : paths) {
+    path_data->paths.append(path);
+    path_data->file_types_bit_flag |= ED_path_extension_type(path);
+    path_data->file_types.append(ED_path_extension_type(path));
+  }
+
+  path_data->tooltip = path_data->paths[0];
+
+  if (path_data->paths.size() > 1) {
+    std::string path_count = std::to_string(path_data->paths.size());
+    path_data->tooltip = fmt::format(fmt::runtime(TIP_("Dragging {} files")), path_count);
+  }
+
   return path_data;
 }
 
 static void wm_drag_free_path_data(wmDragPath **path_data)
 {
-  MEM_freeN((*path_data)->path);
   MEM_delete(*path_data);
   *path_data = nullptr;
 }
 
-const char *WM_drag_get_path(const wmDrag *drag)
+const char *WM_drag_get_single_path(const wmDrag *drag)
 {
   if (drag->type != WM_DRAG_PATH) {
     return nullptr;
   }
 
   const wmDragPath *path_data = static_cast<const wmDragPath *>(drag->poin);
-  return path_data->path;
+  return path_data->paths[0].c_str();
+}
+
+const char *WM_drag_get_single_path(const wmDrag *drag, int file_type)
+{
+  if (drag->type != WM_DRAG_PATH) {
+    return nullptr;
+  }
+  const wmDragPath *path_data = static_cast<const wmDragPath *>(drag->poin);
+  const blender::Span<int> file_types = path_data->file_types;
+
+  const auto *itr = std::find_if(
+      file_types.begin(), file_types.end(), [file_type](const int file_fype_test) {
+        return file_fype_test & file_type;
+      });
+
+  if (itr == file_types.end()) {
+    return nullptr;
+  }
+  const int index = itr - file_types.begin();
+  return path_data->paths[index].c_str();
+}
+
+bool WM_drag_has_path_file_type(const wmDrag *drag, int file_type)
+{
+  if (drag->type != WM_DRAG_PATH) {
+    return false;
+  }
+  const wmDragPath *path_data = static_cast<const wmDragPath *>(drag->poin);
+  return bool(path_data->file_types_bit_flag & file_type);
+}
+
+blender::Span<std::string> WM_drag_get_paths(const wmDrag *drag)
+{
+  if (drag->type != WM_DRAG_PATH) {
+    return blender::Span<std::string>();
+  }
+
+  const wmDragPath *path_data = static_cast<const wmDragPath *>(drag->poin);
+  return path_data->paths.as_span();
 }
 
 int WM_drag_get_path_file_type(const wmDrag *drag)
@@ -791,12 +930,30 @@ int WM_drag_get_path_file_type(const wmDrag *drag)
   }
 
   const wmDragPath *path_data = static_cast<const wmDragPath *>(drag->poin);
-  return path_data->file_type;
+  return path_data->file_types[0];
+}
+
+const std::string &WM_drag_get_string(const wmDrag *drag)
+{
+  BLI_assert(drag->type == WM_DRAG_STRING);
+  const std::string *str = static_cast<const std::string *>(drag->poin);
+  return *str;
+}
+
+std::string WM_drag_get_string_firstline(const wmDrag *drag)
+{
+  BLI_assert(drag->type == WM_DRAG_STRING);
+  const std::string *str = static_cast<const std::string *>(drag->poin);
+  const size_t str_eol = str->find('\n');
+  if (str_eol != std::string::npos) {
+    return str->substr(0, str_eol);
+  }
+  return *str;
 }
 
 /* ************** draw ***************** */
 
-static void wm_drop_operator_draw(const char *name, int x, int y)
+static void wm_drop_operator_draw(const blender::StringRef name, int x, int y)
 {
   const uiFontStyle *fstyle = UI_FSTYLE_WIDGET;
 
@@ -811,7 +968,7 @@ static void wm_drop_operator_draw(const char *name, int x, int y)
   UI_fontstyle_draw_simple_backdrop(fstyle, x, y, name, col_fg, col_bg);
 }
 
-static void wm_drop_redalert_draw(const char *redalert_str, int x, int y)
+static void wm_drop_redalert_draw(const blender::StringRef redalert_str, int x, int y)
 {
   const uiFontStyle *fstyle = UI_FSTYLE_WIDGET;
   const bTheme *btheme = UI_GetTheme();
@@ -845,7 +1002,7 @@ const char *WM_drag_get_item_name(wmDrag *drag)
     }
     case WM_DRAG_PATH: {
       const wmDragPath *path_drag_data = static_cast<const wmDragPath *>(drag->poin);
-      return path_drag_data->path;
+      return path_drag_data->tooltip.c_str();
     }
     case WM_DRAG_NAME:
       return static_cast<const char *>(drag->poin);
@@ -865,6 +1022,11 @@ static int wm_drag_imbuf_icon_height_get(const wmDrag *drag)
   return round_fl_to_int(drag->imb->y * drag->imbuf_scale);
 }
 
+static int wm_drag_preview_icon_size_get()
+{
+  return UI_preview_tile_size_x();
+}
+
 static void wm_drag_draw_icon(bContext * /*C*/, wmWindow * /*win*/, wmDrag *drag, const int xy[2])
 {
   int x, y;
@@ -878,7 +1040,7 @@ static void wm_drag_draw_icon(bContext * /*C*/, wmWindow * /*win*/, wmDrag *drag
     x = xy[0] - (wm_drag_imbuf_icon_width_get(drag) / 2);
     y = xy[1] - (wm_drag_imbuf_icon_height_get(drag) / 2);
 
-    const float col[4] = {1.0f, 1.0f, 1.0f, 0.65f}; /* this blends texture */
+    const float col[4] = {1.0f, 1.0f, 1.0f, 0.65f}; /* This blends texture. */
     IMMDrawPixelsTexState state = immDrawPixelsTexSetup(GPU_SHADER_3D_IMAGE_COLOR);
     immDrawPixelsTexTiled_scaling(&state,
                                   x,
@@ -894,6 +1056,13 @@ static void wm_drag_draw_icon(bContext * /*C*/, wmWindow * /*win*/, wmDrag *drag
                                   1.0f,
                                   col);
   }
+  else if (drag->preview_icon_id) {
+    const int size = wm_drag_preview_icon_size_get();
+    x = xy[0] - (size / 2);
+    y = xy[1] - (size / 2);
+
+    UI_icon_draw_preview(x, y, drag->preview_icon_id, UI_INV_SCALE_FAC, 0.8, size);
+  }
   else {
     int padding = 4 * UI_SCALE_FAC;
     x = xy[0] - 2 * padding;
@@ -912,11 +1081,13 @@ static void wm_drag_draw_item_name(wmDrag *drag, const int x, const int y)
   UI_fontstyle_draw_simple(fstyle, x, y, WM_drag_get_item_name(drag), text_col);
 }
 
-void WM_drag_draw_item_name_fn(bContext * /*C*/, wmWindow * /*win*/, wmDrag *drag, const int xy[2])
+void WM_drag_draw_item_name_fn(bContext * /*C*/, wmWindow *win, wmDrag *drag, const int xy[2])
 {
   int x = xy[0] + 10 * UI_SCALE_FAC;
   int y = xy[1] + 1 * UI_SCALE_FAC;
 
+  /* Needs zero offset here or it looks blurry. #128112. */
+  wmWindowViewport_ex(win, 0.0f);
   wm_drag_draw_item_name(drag, x, y);
 }
 
@@ -928,19 +1099,14 @@ static void wm_drag_draw_tooltip(bContext *C, wmWindow *win, wmDrag *drag, const
   }
   int iconsize = UI_ICON_SIZE;
   int padding = 4 * UI_SCALE_FAC;
-
-  char *tooltip = nullptr;
-  if (drag->drop_state.active_dropbox) {
-    tooltip = dropbox_tooltip(C, drag, xy, drag->drop_state.active_dropbox);
-  }
-
+  blender::StringRef tooltip = drag->drop_state.tooltip;
   const bool has_disabled_info = drag->drop_state.disabled_info &&
                                  drag->drop_state.disabled_info[0];
-  if (!tooltip && !has_disabled_info) {
+  if (tooltip.is_empty() && !has_disabled_info) {
     return;
   }
 
-  const int winsize_y = WM_window_pixels_y(win);
+  const int winsize_y = WM_window_native_pixel_y(win);
   int x, y;
   if (drag->imb) {
     const int icon_width = wm_drag_imbuf_icon_width_get(drag);
@@ -955,6 +1121,18 @@ static void wm_drag_draw_tooltip(bContext *C, wmWindow *win, wmDrag *drag, const
       y = xy[1] - (icon_height / 2) - padding - iconsize - padding - iconsize;
     }
   }
+  if (drag->preview_icon_id) {
+    const int size = wm_drag_preview_icon_size_get();
+
+    x = xy[0] - (size / 2);
+
+    if (xy[1] + (size / 2) + padding + iconsize < winsize_y) {
+      y = xy[1] + (size / 2) + padding;
+    }
+    else {
+      y = xy[1] - (size / 2) - padding - iconsize - padding - iconsize;
+    }
+  }
   else {
     x = xy[0] - 2 * padding;
 
@@ -966,9 +1144,8 @@ static void wm_drag_draw_tooltip(bContext *C, wmWindow *win, wmDrag *drag, const
     }
   }
 
-  if (tooltip) {
+  if (!tooltip.is_empty()) {
     wm_drop_operator_draw(tooltip, x, y);
-    MEM_freeN(tooltip);
   }
   else if (has_disabled_info) {
     wm_drop_redalert_draw(drag->drop_state.disabled_info, x, y);
@@ -988,6 +1165,12 @@ static void wm_drag_draw_default(bContext *C, wmWindow *win, wmDrag *drag, const
     xy_tmp[0] = xy[0] - (wm_drag_imbuf_icon_width_get(drag) / 2);
     xy_tmp[1] = xy[1] - (wm_drag_imbuf_icon_height_get(drag) / 2) - iconsize;
   }
+  else if (drag->preview_icon_id) {
+    const int icon_size = UI_ICON_SIZE;
+    const int preview_size = wm_drag_preview_icon_size_get();
+    xy_tmp[0] = xy[0] - (preview_size / 2);
+    xy_tmp[1] = xy[1] - (preview_size / 2) - icon_size;
+  }
   else {
     xy_tmp[0] = xy[0] + 10 * UI_SCALE_FAC;
     xy_tmp[1] = xy[1] + 1 * UI_SCALE_FAC;
@@ -1050,6 +1233,8 @@ void wm_drags_draw(bContext *C, wmWindow *win)
       CTX_wm_region_set(C, region);
     }
 
+    /* Needs zero offset here or it looks blurry. #128112. */
+    wmWindowViewport_ex(win, 0.0f);
     wm_drag_draw_default(C, win, drag, xy);
   }
   GPU_blend(GPU_BLEND_NONE);
